@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -48,7 +48,7 @@
 
 use strict;
 use warnings;
-use 5.010;
+use 5.26.3;
 
 package RT::Crypt::SMIME;
 
@@ -80,8 +80,8 @@ You should start from reading L<RT::Crypt>.
     Set( %SMIME,
         Enable => 1,
         OpenSSL => '/usr/bin/openssl',
-        Keyring => '/opt/rt5/var/data/smime',
-        CAPath  => '/opt/rt5/var/data/smime/signing-ca.pem',
+        Keyring => '/opt/rt6/var/data/smime',
+        CAPath  => '/opt/rt6/var/data/smime/signing-ca.pem',
         Passphrase => {
             'queue.address@example.com' => 'passphrase',
             'another.queue.address@example.com' => {
@@ -90,7 +90,7 @@ You should start from reading L<RT::Crypt>.
             },
             '' => 'fallback',
         },
-        OtherCertificatesToSend => '/opt/rt5/var/data/smime/other-certs.pem',
+        OtherCertificatesToSend => '/opt/rt6/var/data/smime/other-certs.pem',
         CheckCRL => 0,
         CheckOCSP => 0,
         CheckRevocationDownloadTimeout => 30,
@@ -390,6 +390,8 @@ sub _SignEncrypt {
 
     my $opts = RT->Config->Get('SMIME');
 
+    my @providers = map { ( '-provider', $_ ) } @{ $opts->{'Providers'} };
+
     my @commands;
     if ( $args{'Sign'} ) {
         my $file = $self->CheckKeyring( Key => $args{'Signer'}, For => 'Signing' );
@@ -408,7 +410,10 @@ sub _SignEncrypt {
 
         $args{OtherCertificatesToSend} //= $opts->{OtherCertificatesToSend};
         push @commands, [
-            $self->OpenSSLPath, qw(smime -sign),
+            $self->OpenSSLPath,
+            'smime',
+            @providers,
+            '-sign',
             -signer => $file,
             -inkey  => $file,
             $args{OtherCertificatesToSend} ? ( -certfile => $args{OtherCertificatesToSend} ) : (),
@@ -425,7 +430,7 @@ sub _SignEncrypt {
             $key = $key_file;
         }
         push @commands, [
-            $self->OpenSSLPath, qw(smime -encrypt -des3),
+            $self->OpenSSLPath, qw(smime -encrypt), '-' . ( $opts->{Cipher} || 'aes-128-cbc' ),
             map { $_->filename } @keys
         ];
     }
@@ -488,13 +493,20 @@ sub Verify {
 
     my $msg = $args{'Data'}->as_string;
 
+    my $opts = RT->Config->Get('SMIME');
+
+    my @providers = map { ( '-provider', $_ ) } @{ $opts->{'Providers'} };
+
     my %res;
     my $buf;
     my $keyfh = File::Temp->new;
     {
         local $SIG{CHLD} = 'DEFAULT';
         my $cmd = [
-            $self->OpenSSLPath, qw(smime -verify -noverify),
+            $self->OpenSSLPath,
+            'smime',
+            @providers,
+            qw(-verify -noverify),
             '-signer', $keyfh->filename,
         ];
         safe_run_child { run3( $cmd, \$msg, \$buf, \$res{'stderr'} ) };
@@ -526,7 +538,7 @@ sub Verify {
         $signer = $info{info}[0];
         last unless $signer and $signer->{User}[0]{String};
 
-        unless ( $info{info}[0]{TrustLevel} > 0 or RT->Config->Get('SMIME')->{AcceptUntrustedCAs}) {
+        unless ( $info{info}[0]{TrustLevel} > 0 or $opts->{AcceptUntrustedCAs}) {
             # We don't trust it; give it the finger
             $res{exit_code} = 1;
             $res{'message'} = "Validation failed";
@@ -647,12 +659,16 @@ sub _Decrypt {
         grep !$seen{lc $_}++, map $_->address, map Email::Address->parse($_),
         grep length && defined, @{$args{'Recipients'}};
 
+    my $opts = RT->Config->Get('SMIME');
+
+    my @providers = map { ( '-provider', $_ ) } @{ $opts->{'Providers'} };
+
     my ($buf, $encrypted_to, %res);
 
     foreach my $address ( @addresses ) {
         my $file = $self->CheckKeyring( Key => $address, For => 'Encryption' );
         unless ( $file ) {
-            my $keyring = RT->Config->Get('SMIME')->{'Keyring'};
+            my $keyring = $opts->{'Keyring'};
             $RT::Logger->debug("No key found for $address in $keyring directory");
             next;
         }
@@ -661,7 +677,9 @@ sub _Decrypt {
         local $SIG{CHLD} = 'DEFAULT';
         my $cmd = [
             $self->OpenSSLPath,
-            qw(smime -decrypt),
+            'smime',
+            @providers,
+            '-decrypt',
             -recip => $file,
             (defined $ENV{'SMIME_PASS'} && length $ENV{'SMIME_PASS'})
                 ? (qw(-passin env:SMIME_PASS))
@@ -679,6 +697,12 @@ sub _Decrypt {
             next;
         }
 
+        if ( index( $res{'stderr'}, 'no recipient matches certificate' ) >= 0 ) {
+            $RT::Logger->debug(
+                "Although we have a certificate for $address, it is not the one that encrypted this message");
+            next;
+        }
+
         $res{'exit_code'} = $?;
         $res{'message'} = "openssl exited with error code ". ($? >> 8)
             ." and error: $res{stderr}";
@@ -1246,18 +1270,22 @@ sub CheckRevocationUsingOCSP {
     $out = '';
     $err = '';
 
-    safe_run_child { run3( [$self->OpenSSLPath(), 'ocsp', '-issuer', $issuer, '-cert', '-', @ca_verify, '-url', $ocsp_url],
-                           \$PEM, \$out, \$err) };
+    my $cert_fh = File::Temp->new;
+    print $cert_fh $PEM;
+    close $cert_fh;
+    my $cert_file = $cert_fh->filename;
+    safe_run_child { run3( [$self->OpenSSLPath(), 'ocsp', '-issuer', $issuer, '-cert', $cert_file, @ca_verify, '-url', $ocsp_url],
+                           undef, \$out, \$err) };
     return undef unless $? == 0;
 
-    if ($out =~ /^-: revoked/) {
+    if ($out =~ /^\Q$cert_file\E: revoked/) {
         $res->{info}[0]{Trust} = "REVOKED certificate checked against OCSP URI $ocsp_url";
         $res->{info}[0]{TrustTerse} = "none (revoked certificate)";
         $res->{info}[0]{TrustLevel} = -1;
         $res->{exit_code} = 0;
         return 1;
     }
-    if ($out =~ /^-: good/) {
+    if ($out =~ /^\Q$cert_file\E: good/) {
         # Definitely NOT revoked.  Return 0, but not undef
         return 0;
     }
@@ -1321,4 +1349,6 @@ sub GetCertificateForTransaction {
     return $out;
 }
 
+RT::Base->_ImportOverlays();
+
 1;
