@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2024 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -228,6 +228,9 @@ sub PushAll {
     # Scrips
     $self->PushCollections(qw(ScripActions ScripConditions Templates Scrips ObjectScrips));
 
+    # SavedSearches and Dashboards
+    $self->PushCollections(qw(SavedSearches Dashboards DashboardSubscriptions ObjectContents));
+
     # Attributes
     $self->PushCollections(qw(Attributes));
 
@@ -629,4 +632,6 @@ sub Visit {
     $self->{ObjectCount}{$store[0]}++;
 }
 
+RT::Base->_ImportOverlays();
+
 1;
