@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -69,7 +69,7 @@ use warnings;
 use RT;
 use base RT->Config->Get('RecordBaseClass');
 use base 'RT::Base';
-use v5.10;
+use 5.26.3;
 
 require RT::Date;
 require RT::User;
@@ -263,7 +263,7 @@ If any Column has a Validate$PARAMNAME subroutine defined and the
 value provided doesn't pass validation, this routine returns
 an error.
 
-If this object's table has any of the following atetributes defined as
+If this object's table has any of the following attributes defined as
 'Auto', this routine will automatically fill in their values.
 
 =over
@@ -320,7 +320,14 @@ sub Create {
     $attribs{'LastAccessedBy'} = $self->CurrentUser->id || '0'
       if ( $self->_Accessible( 'LastAccessedBy', 'auto' ) && !$attribs{'LastAccessedBy'});
 
-    my $id = $self->SUPER::Create(%attribs);
+    # Do not filter %attribs if the class's columns are not defined at all.
+    my $id = $self->SUPER::Create(
+        map {
+                  ( !keys %{ $self->_ClassAccessible || {} } || $self->_ClassAccessible->{$_} )
+                ? ( $_ => $attribs{$_} )
+                : ()
+        } keys %attribs
+    );
     if ( UNIVERSAL::isa( $id, 'Class::ReturnValue' ) ) {
         if ( $id->errno ) {
             if (wantarray) {
@@ -743,7 +750,7 @@ sub _EncodeLOB {
     #if the current attachment contains nulls and the
     #database doesn't support embedded nulls
 
-    if ( ( !$RT::Handle->BinarySafeBLOBs ) && ( $Body =~ /\x00/ ) ) {
+    if ( ( !$RT::Handle->BinarySafeBLOBs ) && ( $Body =~ /\x00/ || length $Body > 100_000 ) ) {
 
         # set a flag telling us to mimencode the attachment
         $ContentEncoding = 'base64';
@@ -1010,7 +1017,7 @@ sub _UpdateAttributes {
                                    "The new value has been set.",          # loc
                                    "No column specified",                  # loc
                                    "Immutable field",                      # loc
-                                   "Nonexistant field?",                   # loc
+                                   "Nonexistent field?",                   # loc
                                    "Invalid data",                         # loc
                                    "Couldn't find row",                    # loc
                                    "Missing a primary key?: [_1]",         # loc
@@ -1182,6 +1189,70 @@ sub AllDependsOn {
                                      Direction => 'Base', @_ );
 }
 
+=head2 AllMembers
+
+Returns an array of RT::Ticket objects which (directly or indirectly)
+are members of this ticket; takes an optional 'Type' argument in the param
+hash, which will limit returned tickets to that type, as well as cause
+tickets with that type to serve as 'leaf' nodes that stops the recursive
+dependency search.
+
+=cut
+
+sub AllMembers {
+    my $self = shift;
+    return $self->_AllLinkedTickets( LinkType => 'MemberOf',
+                                     Direction => 'Target', @_ );
+}
+
+=head2 AllMembersOf
+
+Returns an array of RT::Ticket objects which this ticket (directly or
+indirectly) is a member of; takes an optional 'Type' argument in the param
+hash, which will limit returned tickets to that type, as well as cause
+tickets with that type to serve as 'leaf' nodes that stops the
+recursive dependency search.
+
+=cut
+
+sub AllMembersOf {
+    my $self = shift;
+    return $self->_AllLinkedTickets( LinkType => 'MemberOf',
+                                     Direction => 'Base', @_ );
+}
+
+=head2 AllReferredToBy
+
+Returns an array of RT::Ticket objects which (directly or indirectly)
+are referred to by this ticket; takes an optional 'Type' argument in the param
+hash, which will limit returned tickets to that type, as well as cause
+tickets with that type to serve as 'leaf' nodes that stops the recursive
+dependency search.
+
+=cut
+
+sub AllReferredToBy {
+    my $self = shift;
+    return $self->_AllLinkedTickets( LinkType => 'RefersTo',
+                                     Direction => 'Target', @_ );
+}
+
+=head2 AllRefersTo
+
+Returns an array of RT::Ticket objects which this ticket (directly or
+indirectly) refers to; takes an optional 'Type' argument in the param
+hash, which will limit returned tickets to that type, as well as cause
+tickets with that type to serve as 'leaf' nodes that stops the
+recursive dependency search.
+
+=cut
+
+sub AllRefersTo {
+    my $self = shift;
+    return $self->_AllLinkedTickets( LinkType => 'RefersTo',
+                                     Direction => 'Base', @_ );
+}
+
 sub _AllLinkedTickets {
     my $self = shift;
 
@@ -1411,7 +1482,19 @@ sub _AddLink {
             LinkType  => $args{'Type'},
             Direction => $direction eq 'Target' ? 'Base' : 'Target',
         );
-        if ( grep { $_->id == ( $direction eq 'Target' ? $args{'Base'} : $args{'Target'} ) } @tickets ) {
+
+        my $uri_ok;
+        my $local_uri = RT::URI->new( RT->SystemUser );
+        if ( $direction eq 'Target' ) {
+            $uri_ok = $local_uri->FromURI($args{'Base'});
+            return ( 0, $self->loc("Unable to parse link base") . ' ' . $args{'Base'} ) unless $uri_ok;
+        }
+        else {
+            $uri_ok = $local_uri->FromURI($args{'Target'});
+            return ( 0, $self->loc("Unable to parse link target") . ' ' . $args{'Target'} ) unless $uri_ok;
+        }
+
+        if ( $local_uri->IsLocal && grep { $_->id == $local_uri->Object->Id } @tickets ) {
             return ( 0, $self->loc("Refused to add link which would create a circular relationship") );
         }
     }
@@ -1632,6 +1715,8 @@ sub _NewTransaction {
     my $self = shift;
     my %args = (
         TimeTaken => undef,
+        TimeWorker     => undef,
+        TimeWorkedDate => undef,
         Type      => undef,
         OldValue  => undef,
         NewValue  => undef,
@@ -1670,6 +1755,8 @@ sub _NewTransaction {
         ObjectId  => $self->Id,
         ObjectType => ref($self),
         TimeTaken => $args{'TimeTaken'},
+        TimeWorker     => $args{'TimeWorker'},
+        TimeWorkedDate => $args{'TimeWorkedDate'},
         Type      => $args{'Type'},
         Data      => $args{'Data'},
         Field     => $args{'Field'},
@@ -1783,6 +1870,7 @@ our %TRANSACTION_CLASSIFICATION = (
         map( { $_ => 'people' } qw(
             Owner Creator LastUpdatedBy
         ) ),
+        'Description' => 'description',
     },
     CustomField => 'cfs',
     SystemError => 'error',
@@ -2636,20 +2724,26 @@ sub CustomDateRange {
         my $schedule;
         my $timezone;
 
-        # Prefer the schedule/timezone specified in %ServiceAgreements for current object
-        if ( $self->isa('RT::Ticket') && !$self->QueueObj->SLADisabled && $self->SLA ) {
-            if ( my $config = RT->Config->Get('ServiceAgreements') ) {
-                if ( ref( $config->{QueueDefault}{ $self->QueueObj->Name } ) eq 'HASH' ) {
-                    $timezone = $config->{QueueDefault}{ $self->QueueObj->Name }{Timezone};
-                }
+        if ( $date_range_spec{business_time} eq '1' ) {
+            # Prefer the schedule/timezone specified in %ServiceAgreements for current object
+            if ( $self->isa('RT::Ticket') && !$self->QueueObj->SLADisabled && $self->SLA ) {
+                if ( my $config = RT->Config->Get('ServiceAgreements') ) {
+                    if ( ref( $config->{QueueDefault}{ $self->QueueObj->Name } ) eq 'HASH' ) {
+                        $timezone = $config->{QueueDefault}{ $self->QueueObj->Name }{Timezone};
+                    }
 
-                # Each SLA could have its own schedule and timezone
-                if ( my $agreement = $config->{Levels}{ $self->SLA } ) {
-                    $schedule = $agreement->{BusinessHours};
-                    $timezone ||= $agreement->{Timezone};
+                    # Each SLA could have its own schedule and timezone
+                    if ( my $agreement = $config->{Levels}{ $self->SLA } ) {
+                        $schedule = $agreement->{BusinessHours};
+                        $timezone ||= $agreement->{Timezone};
+                    }
                 }
             }
         }
+        else {
+            $schedule = $date_range_spec{business_time};
+        }
+
         $timezone ||= RT->Config->Get('Timezone');
         $schedule ||= 'Default';
 
@@ -2853,6 +2947,22 @@ sub FindDependencies {
         $objs->LimitToObject( $self );
         $deps->Add( in => $objs );
     }
+
+    for my $collection (qw/SavedSearches Dashboards/) {
+        next unless $self->can($collection);
+        my ($objs) = $self->$collection;
+        next unless $objs;
+        if ( $objs->isa('RT::SearchBuilder') ) {
+            $objs->FindAllRows;
+            $deps->Add( in => $objs );
+        }
+    }
+
+    $objs = RT::ObjectContents->new( $self->CurrentUser );
+    $objs->FindAllRows;
+    $objs->Limit( FIELD => 'ObjectType', VALUE => ref $self );
+    $objs->Limit( FIELD => 'ObjectId', VALUE => $self->Id );
+    $deps->Add( in => $objs );
 }
 
 sub Serialize {
@@ -2951,6 +3061,10 @@ sub Serialize {
         }
         delete $store{$_} for qw/ObjectType ObjectId/;
     }
+    elsif ( $store{ObjectId} && $self->can('Object') && $self->Object->_Accessible( 'Name', 'read' ) ) {
+        $store{ObjectId} = $self->Object->Name;
+    }
+
 
     return %store;
 }
@@ -3032,7 +3146,8 @@ sub _AsInsertQuery
 
     my $dbh = $RT::Handle->dbh;
 
-    my $res = "INSERT INTO ". $self->Table;
+    my $table = $self->can('QuotedTableName') ? $self->QuotedTableName($self->Table) : $self->Table;
+    my $res = "INSERT INTO $table";
     my $values = $self->{'values'};
     $res .= "(". join( ",", map { $dbh->quote_identifier( $_ ) } sort keys %$values ) .")";
     $res .= " VALUES";
@@ -3042,7 +3157,66 @@ sub _AsInsertQuery
     return $res;
 }
 
-sub BeforeWipeout { return 1 }
+sub BeforeWipeout {
+    my $self = shift;
+    if (   $RT::Shredder::IncludeExternalStorage
+        && ( $self->isa('RT::Attachment') || $self->isa('RT::ObjectCustomFieldValue') )
+        && ( $self->ContentEncoding // '' ) eq 'external' )
+    {
+        my $digest = $self->ExternalStoreDigest;
+
+        my $storage = RT->System->ExternalStorage;
+        unless ($storage) {
+            RT->Logger->error("External storage not configured");
+            RT::Shredder::Exception::Info->throw('InvalidExternalStorage');
+        }
+
+        # Internalize content so we can re-create objects easily from generated SQL
+        if ( my $external_content = $storage->Get($digest) ) {
+            # only RT::Attachment records have a Filename
+            my @encode_lob_args = ( $external_content, $self->ContentType );
+            push @encode_lob_args, $self->Filename
+                if $self->isa('RT::Attachment');
+            my ( $encoding, $content ) = $self->_EncodeLOB(@encode_lob_args);
+            my ( $ret,      $msg )     = $self->__Set( Field => 'ContentEncoding', Value => $encoding );
+            if ( !$ret ) {
+                RT->Logger->error("Could not set ContentEncoding to $encoding: $msg");
+                RT::Shredder::Exception::Info->throw('CouldntInternalizeObject');
+            }
+
+            ( $ret, $msg ) = $self->__Set( Field => 'Content', Value => $content );
+            if ( !$ret ) {
+                RT->Logger->error("Could not set Content: $msg");
+                RT::Shredder::Exception::Info->throw('CouldntInternalizeObject');
+            }
+            $self->{_internalized} = 1;
+        }
+        else {
+            # Can't internalize since content is absent somehow
+            RT->Logger->error("Could not get content of $digest");
+            return 1;
+        }
+
+        # Delete external resource only if there are no objects referenced to it
+        for my $class (qw/RT::Attachments RT::ObjectCustomFieldValues/) {
+            my $objects = $class->new( $self->CurrentUser );
+            $objects->Limit( FIELD => 'ContentEncoding', VALUE => 'external' );
+            $objects->Limit( FIELD => 'Content',         VALUE => $digest );
+            $objects->RowsPerPage(1);
+            if ( $objects->First ) {
+                RT->Logger->info("$digest is referenced by other objects, skipping");
+                return 1;
+            }
+        }
+
+        my ( $ret, $msg ) = $storage->Delete($digest);
+        if ( !$ret ) {
+            RT->Logger->error("Failed to delete $digest from external storage: $msg");
+            RT::Shredder::Exception::Info->throw('CouldntDeleteExternalObject');
+        }
+    }
+    return 1;
+}
 
 =head2 Dependencies
 
@@ -3121,6 +3295,13 @@ sub __DependsOn
     $objs->LimitToObject( $self );
     push( @$list, $objs );
 
+    # Object contents
+    $objs = RT::ObjectContents->new( $self->CurrentUser );
+    $objs->FindAllRows;
+    $objs->Limit( FIELD => 'ObjectType', VALUE => ref $self );
+    $objs->Limit( FIELD => 'ObjectId',   VALUE => $self->Id );
+    push @$list, $objs;
+
     $deps->_PushDependencies(
             BaseObject => $self,
             Flags => RT::Shredder::Constants::DEPENDS_ON,
