@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2024 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -97,6 +97,20 @@ sub LimitToType  {
 
 }
 
+=head2 LimitToLookupType LOOKUPTYPE
+
+Takes LookupType and limits collection.
+
+=cut
+
+sub LimitToLookupType {
+    my $self   = shift;
+    my $lookup = shift;
+
+    $self->Limit( FIELD => 'LookupType', VALUE => $lookup );
+}
+
+
 RT::Base->_ImportOverlays();
 
 1;
