@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2024 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -75,6 +75,8 @@ use base 'RT::Record';
 use strict;
 use warnings;
 
+use Role::Basic 'with';
+with "RT::Record::Role::ObjectContent" => { -rename   => { SetContent => '_SetContent', Content => '_Content', ContentObj => '_ContentObj' } };
 
 use vars qw( %_BriefDescriptions $PreferredContentType );
 
@@ -84,6 +86,7 @@ use RT::Ruleset;
 
 use HTML::FormatText::WithLinks::AndTables;
 use HTML::Scrubber;
+use Encode;
 
 # For EscapeHTML() and decode_entities()
 require RT::Interface::Web;
@@ -97,12 +100,31 @@ sub Table {'Transactions'}
 
 Create a new transaction.
 
-This routine should _never_ be called by anything other than RT::Ticket. 
-It should not be called 
-from client code. Ever. Not ever.  If you do this, we will hunt you down and break your kneecaps.
-Then the unpleasant stuff will start.
+This method should almost never be called directly. Wrappers are
+available for some cases. See L<RT::Record/_NewTransaction>.
+
+Accepts the following parameters:
+
+    TimeTaken -- Time spent on this transaction in minutes
+    TimeWorker -- User id of the user who worked the time,
+                  can be different from Creator
+    TimeWorkedDate -- Date the recorded time was worked, can be different
+                      from Created
+    Type -- Type of this transaction
+    Data -- For recorded email, the message id of this transaction
+    Field -- The field tracked by this transaction
+    OldValue -- Previous value for this field
+    NewValue -- Current (new) value for this field
+    MIMEObj -- MIME object associated with this transaction
+    ActivateScrips -- Should scrips run for this transaction?
+    DryRun -- Pass DryRun to scrips, skipping the commit phase
+    ObjectType -- Object this transaction applies to, like 'RT::Ticket'
+    ObjectId -- The Id this transaction applies to
+    ReferenceType -- Type of reference, if Old/New Value are a reference
+    OldReference -- Id reference for the old value tracked by this transaction
+    NewReference -- Id reference for the new value tracked by this transaction
+    SquelchMailTo -- One or more email addresses to not send to on this transaction
 
-TODO: Document what gets passed to this
 
 =cut
 
@@ -111,6 +133,8 @@ sub Create {
     my %args = (
         id             => undef,
         TimeTaken      => 0,
+        TimeWorker     => undef,
+        TimeWorkedDate => undef,
         Type           => 'undefined',
         Data           => '',
         Field          => undef,
@@ -153,10 +177,54 @@ sub Create {
     );
 
     # Parameters passed in during an import that we probably don't want to touch, otherwise
-    foreach my $attr (qw(id Creator Created LastUpdated TimeTaken LastUpdatedBy)) {
+    foreach my $attr (qw(id Creator Created LastUpdated TimeTaken TimeWorker TimeWorkedDate LastUpdatedBy)) {
         $params{$attr} = $args{$attr} if ($args{$attr});
     }
- 
+
+    if ( $params{'TimeWorkedDate'} ) {
+        # Make sure TimeWorkedDate is date only, no time
+        my $time_worked_obj = RT::Date->new( RT->SystemUser );
+        $time_worked_obj->Set( Format => 'date', Value => $params{'TimeWorkedDate'} );
+
+        unless ( $time_worked_obj->IsSet ) {
+            RT->Logger->error("Invalid TimeWorkedDate format: " . $params{'TimeWorkedDate'} . " Defaulting to today");
+            # Set to 0 so it will get set to today below.
+            $params{'TimeWorkedDate'} = 0;
+        }
+    }
+
+    # If TimeTaken is provided, but no worker or date, default
+    # to current user and now.
+    if ( $params{'TimeTaken'} ) {
+        $params{'TimeWorker'} ||= $self->CurrentUser->Id;
+    }
+
+    if ( $params{'TimeTaken'} && !$params{'TimeWorkedDate'} ) {
+        my $date = RT::Date->new( $self->CurrentUser );
+        $date->SetToNow();
+        $params{'TimeWorkedDate'} = $date->Date( Timezone => 'user' );
+    }
+
+    my $max_length = 0;
+    for my $field ( qw/OldValue NewValue/ ) {
+        next unless $params{$field};
+        my $length = length encode( 'UTF-8', $params{$field} );
+        $max_length = $length if $max_length < $length;
+    }
+
+    my %content;
+    if ( $max_length > 255 ) {
+        if ( $params{'ReferenceType'} ) {
+            RT->Logger->error("Long OldValue/NewValue and ReferenceType can not coexist");
+            return (0, $self->loc('Long OldValue/NewValue and ReferenceType can not coexist'));
+        }
+        else {
+            $params{'ReferenceType'} = 'RT::ObjectContent';
+            $content{$_} = delete $params{$_} for qw/OldValue NewValue/;
+        }
+    }
+
+
     my $id = $self->SUPER::Create(%params);
     $self->Load($id);
     if ( defined $args{'MIMEObj'} ) {
@@ -167,6 +235,11 @@ sub Create {
         }
     }
 
+    if (%content) {
+        my ( $ret, $msg ) = $self->_SetContent( \%content, RecordTransaction => 0 );
+        return ( 0, $msg ) unless $ret;
+    }
+
     $self->AddAttribute(
         Name    => 'SquelchMailTo',
         Content => RT::User->CanonicalizeEmailAddress($_)
@@ -174,7 +247,11 @@ sub Create {
 
     my @return = ( $id, $self->loc("Transaction Created") );
 
-    return @return unless $args{'ObjectType'} eq 'RT::Ticket';
+    # Are scrips enabled for this object type?
+    my ($lookup_type) = grep
+        { $args{'ObjectType'} eq RT::Scrip->ObjectTypeFromLookupType( $_ ) }
+        RT::Scrip->LookupTypes;
+    return @return unless $lookup_type;
 
     # Provide a way to turn off scrips if we need to
     unless ( $args{'ActivateScrips'} ) {
@@ -191,21 +268,23 @@ sub Create {
     $self->{'scrips'}->Prepare(
         Stage       => 'TransactionCreate',
         Type        => $args{'Type'},
-        Ticket      => $args{'ObjectId'},
+        ObjectId    => $args{'ObjectId'},
+        LookupType  => $lookup_type,
         Transaction => $self->id,
     );
 
    # Entry point of the rule system
-   my $ticket = RT::Ticket->new(RT->SystemUser);
-   $ticket->Load($args{'ObjectId'});
+   my $object = $args{'ObjectType'}->new(RT->SystemUser);
+   $object->Load($args{'ObjectId'});
    my $txn = RT::Transaction->new($RT::SystemUser);
    $txn->Load($self->id);
 
    my $rules = $self->{rules} = RT::Ruleset->FindAllRules(
-        Stage       => 'TransactionCreate',
-        Type        => $args{'Type'},
-        TicketObj   => $ticket,
-        TransactionObj => $txn,
+        Stage                       => 'TransactionCreate',
+        Type                        => $args{'Type'},
+        Object                      => $object,
+        $object->RecordType . 'Obj' => $object,
+        TransactionObj              => $txn,
    );
 
     unless ($args{DryRun} ) {
@@ -379,6 +458,13 @@ sub Content {
             if ($args{Type} ne 'text/html') {
                 $content = RT::Interface::Email::ConvertHTMLToText($content);
             } else {
+                if ( $content =~ /<style.*>/ ) {
+                    require CSS::Inliner;
+                    my $css_inliner = CSS::Inliner->new( { encode_entities => 1, ignore_style_type_attr => 1 } );
+                    $css_inliner->read( { html => $content } );
+                    $content = $css_inliner->inlinify();
+                }
+
                 # Scrub out <html>, <head>, <meta>, and <body>, and
                 # leave all else untouched.
                 my $scrubber = HTML::Scrubber->new();
@@ -711,6 +797,14 @@ sub _FindPreferredContentObj {
             }
 
         }
+
+        # Handle the case where multipart/related is a child of multipart/alternative.
+        my $related_parts = $Attachment->Children;
+        $related_parts->ContentType( VALUE => 'multipart/related' );
+        while ( my $child = $related_parts->Next ) {
+            my $ret = _FindPreferredContentObj( %args, Attachment => $child );
+            return $ret if $ret;
+        }
     }
 
     # If this is a message/rfc822 mail, we need to dig into it in order to find 
@@ -1087,11 +1181,25 @@ sub _CanonicalizeRoleName {
     },
     Correspond => sub {
         my $self = shift;
-        return ("Correspondence added");    #loc()
+
+        if ( my $worked_date = $self->TimeWorkedDate ) {
+            if ( $self->CreatedObj->Date( Timezone => 'user' ) ne $worked_date ) {
+                return ( "Correspondence added for [_1]", $self->TimeWorkedDateAsString );    #loc()
+            }
+        }
+
+        return ("Correspondence added"); #loc()
     },
     Comment => sub {
         my $self = shift;
-        return ("Comments added");          #loc()
+
+        if ( my $worked_date = $self->TimeWorkedDate ) {
+            if ( $self->CreatedObj->Date( Timezone => 'user' ) ne $worked_date ) {
+                return ( "Comments added for [_1]", $self->TimeWorkedDateAsString );    #loc()
+            }
+        }
+
+        return ("Comments added"); #loc()
     },
     CustomField => sub {
         my $self = shift;
@@ -1385,6 +1493,21 @@ sub _CanonicalizeRoleName {
 
             return ( "[_1] changed from [_2] to [_3]", $self->loc( $self->Field ), "'$old_value'", "'$new_value'" );    #loc()
         }
+        elsif ( $self->ObjectType eq 'RT::Ticket' && $self->Field eq 'Description' ) {
+            my $field = $self->Field;
+            my $old = $self->OldValue // '';
+            my $new = $self->NewValue // '';
+            if ( !$old ) {
+                return ( "[_1] added", $field );    #loc()
+            }
+            elsif ( !$new ) {
+                return ( "[_1] deleted", $field );    #loc()
+            }
+            else {
+                return ( "[_1] changed", $field );    #loc()
+            }
+
+        }
         else {
             return ( "[_1] changed from [_2] to [_3]",
                     $self->loc($self->Field),
@@ -1393,17 +1516,42 @@ sub _CanonicalizeRoleName {
         }
     },
     "Set-TimeWorked" => sub {
-        my $self = shift;
-        my $old  = $self->OldValue || 0;
-        my $new  = $self->NewValue || 0;
+        my $self     = shift;
+        my $old      = $self->OldValue || 0;
+        my $new      = $self->NewValue || 0;
         my $duration = $new - $old;
+        my $creator  = $self->Creator;
+        my $worker   = $self->TimeWorker;
+        my $worker_name;
+
+        if ( $worker && $worker != $creator ) {
+            my $user_obj = RT::User->new($self->CurrentUser);
+            $user_obj->Load($worker);
+            $worker_name = $user_obj->Name if $user_obj->Id;
+        }
+
+        my $worked_date_str = $self->TimeWorkedDateAsString;
+
         if ($duration < 0) {
-            return ("Adjusted time worked by [quant,_1,minute,minutes]", $duration); # loc()
+            if ( $worker_name ) {
+                return ("Adjusted time worked by [quant,_1,minute,minutes] on [_2] for [_3]", $duration, $worked_date_str, $worker_name); # loc()
+            }
+            else {
+                return ("Adjusted time worked by [quant,_1,minute,minutes] on [_2]", $duration, $worked_date_str); # loc()
+            }
         }
         elsif ($duration < 60) {
-            return ("Worked [quant,_1,minute,minutes]", $duration); # loc()
+            if ( $worker_name ) {
+                return ("[_1] worked [quant,_2,minute,minutes] on [_3]", $worker_name, $duration, $worked_date_str); # loc()
+            } else {
+                return ("Worked [quant,_1,minute,minutes] on [_2]", $duration, $worked_date_str); # loc()
+            }
         } else {
-            return ("Worked [quant,_1,hour,hours] ([quant,_2,minute,minutes])", sprintf("%.2f", $duration / 60), $duration); # loc()
+            if ( $worker_name ) {
+               return ("[_1] worked [quant,_2,hour,hours] ([quant,_3,minute,minutes]) on [_4]", $worker_name, sprintf("%.2f", $duration / 60), $duration, $worked_date_str); # loc()
+            } else {
+                return ("Worked [quant,_1,hour,hours] ([quant,_2,minute,minutes]) on [_3]", sprintf("%.2f", $duration / 60), $duration, $worked_date_str); # loc()
+            }
         }
     },
     PurgeTransaction => sub {
@@ -1564,6 +1712,18 @@ sub _CanonicalizeRoleName {
             $principal->DisplayName,
         );    #loc()
     },
+    'Set-Image' => sub {
+        my $self = shift;
+
+        my $old    = $self->OldValue;
+        my $new    = $self->NewValue;
+        if ( $new ) {
+            return $old ? 'Image updated' : 'Image added';
+        }
+        else {
+            return 'Image deleted';
+        }
+    },
 
 );
 
@@ -1673,11 +1833,36 @@ sub TicketObj {
     return $self->Object;
 }
 
+sub TimeWorkedDate {
+    my $self = shift;
+    my $date = $self->_Value('TimeWorkedDate');
+    $date =~ s! .+!! if $date; # Oracle's date field contains time
+    return $date;
+}
+
+sub TimeWorkedDateObj {
+    my $self = shift;
+    my $obj  = RT::Date->new( $self->CurrentUser );
+
+    $obj->Set( Format => 'date', Value => $self->TimeWorkedDate );
+
+    return $obj;
+}
+
+sub TimeWorkedDateAsString {
+    my $self = shift;
+    return unless $self->TimeWorkedDate;
+    return $self->TimeWorkedDateObj->AsString( Time => 0, Timezone => 'UTC' );
+}
+
 sub OldValue {
     my $self = shift;
     if ( my $Object = $self->OldReferenceObject ) {
         return $Object->Content;
     }
+    elsif ( ( $self->ReferenceType // '' ) eq 'RT::ObjectContent' ) {
+        return ( $self->_Content || {} )->{OldValue};
+    }
     else {
         return $self->_Value('OldValue');
     }
@@ -1688,6 +1873,9 @@ sub NewValue {
     if ( my $Object = $self->NewReferenceObject ) {
         return $Object->Content;
     }
+    elsif ( ( $self->ReferenceType // '' ) eq 'RT::ObjectContent' ) {
+        return ( $self->_Content || {} )->{NewValue};
+    }
     else {
         return $self->_Value('NewValue');
     }
@@ -2006,6 +2194,26 @@ Returns the current value of TimeTaken.
 
 
 
+=head2 TimeWorker
+
+Returns the current value of TimeWorker.
+(In the database, TimeWorker is stored as int(11).)
+
+=head2 TimeWorkedDate
+
+Returns the current value of TimeWorkedDate as a date in
+the format YYYY-MM-DD.
+
+=head2 TimeWorkedDateAsString
+
+Returns the current value of TimeWorkedDate as a date in the format of
+current user's preference.
+
+=head2 TimeWorkedDateObj
+
+Returns an RT::Date object set with the current value of
+TimeWorkedDate.
+
 =head2 SetTimeTaken VALUE
 
 
@@ -2191,6 +2399,10 @@ sub _CoreAccessible {
                 {read => 1, write => 1, sql_type => 4, length => 11,  is_blob => 0,  is_numeric => 1,  type => 'int(11)', default => '0'},
         TimeTaken =>
                 {read => 1, write => 1, sql_type => 4, length => 11,  is_blob => 0,  is_numeric => 1,  type => 'int(11)', default => '0'},
+        TimeWorker =>
+                {read => 1, auto => 1, sql_type => 4, length => 11,  is_blob => 0,  is_numeric => 1,  type => 'int(11)', default => '0'},
+        TimeWorkedDate =>
+                {read => 1, auto => 1, sql_type => 10, length => 0,  is_blob => 0,  is_numeric => 0,  type => 'date', default => ''},
         Type =>
                 {read => 1, write => 1, sql_type => 12, length => 20,  is_blob => 0,  is_numeric => 0,  type => 'varchar(20)', default => ''},
         Field =>
@@ -2266,6 +2478,12 @@ sub FindDependencies {
         $ticket->Load( $self->NewValue );
         $deps->Add( out => $ticket );
     }
+
+    if ( $self->TimeWorker && $self->TimeWorker != $self->Creator ) {
+        my $user = RT::User->new( RT->SystemUser );
+        $user->Load( $self->TimeWorker );
+        $deps->Add( out => $user );
+    }
 }
 
 sub __DependsOn {
@@ -2371,6 +2589,10 @@ sub Serialize {
         $store{NewValue} = \( join '-', 'RT::Ticket', $RT::Organization, $store{NewValue} );
     }
 
+    if ( $store{TimeWorker} && $store{TimeWorker} != $store{Creator} ) {
+        $store{TimeWorker} = \$args{serializer}{_uid}{user}{ $store{TimeWorker} };
+    }
+
     return %store;
 }
 
@@ -2396,6 +2618,26 @@ sub PreInflate {
     return $class->SUPER::PreInflate( $importer, $uid, $data );
 }
 
+=head2 TimeWorkerObj
+
+Returns an RT::User object with the RT account of the TimeWorker of this row
+
+=cut
+
+sub TimeWorkerObj {
+    my $self = shift;
+    unless ( exists $self->{'TimeWorkerObj'} ) {
+        if ( $self->TimeTaken == 0 ) {
+            $self->{'TimeWorkerObj'} = undef;
+        }
+        else {
+            $self->{'TimeWorkerObj'} = RT::User->new( $self->CurrentUser );
+            $self->{'TimeWorkerObj'}->Load( $self->TimeWorker );
+        }
+    }
+    return ( $self->{'TimeWorkerObj'} );
+}
+
 RT::Base->_ImportOverlays();
 
 1;
